@@ -45,7 +45,7 @@ class DataEncoder(DataPipelineSegment):
                     apply_value_representation=True,
                     _add_to_que=True):
         """
-        Desc:
+
             Encode the data into numerical values for machine learning processes.
 
         Args:
@@ -111,7 +111,7 @@ class DataEncoder(DataPipelineSegment):
                     apply_value_representation=True,
                     _add_to_que=True):
         """
-        Desc:
+
             Decode the data into non-numerical values for more descriptive analysis.
 
         Args:
@@ -204,7 +204,7 @@ class DataEncoder(DataPipelineSegment):
                                    df_features,
                                    _add_to_que=True):
         """
-        Desc:
+
             Translate features into most understandable/best representation
 
         Args:
@@ -242,7 +242,7 @@ class DataEncoder(DataPipelineSegment):
                                     df_features,
                                     _add_to_que=True):
         """
-        Desc:
+
             Translate features back into worst representation
 
         Args:
@@ -283,7 +283,7 @@ class DataEncoder(DataPipelineSegment):
                          df_features,
                          _add_to_que=True):
         """
-        Desc:
+
             Convert all string bools to numeric bool value
         Args:
             df: pd.Dataframe
@@ -320,7 +320,7 @@ class DataEncoder(DataPipelineSegment):
                      _feature_values_dict=None,
                      _add_to_que=True):
         """
-        Desc:
+
             Create dummies features of based on qualtative feature data and removes
             the original feature.
 
@@ -395,7 +395,7 @@ class DataEncoder(DataPipelineSegment):
                        qualitative_features=[],
                        _add_to_que=True):
         """
-        Desc:
+
             Convert dummies features back to the original feature.
 
         Args:
@@ -414,48 +414,41 @@ class DataEncoder(DataPipelineSegment):
                 to the pipeline segment.
         """
 
-        old_indexes = copy.deepcopy(df.index)
-
         df.reset_index(inplace=True,
                        drop=True)
 
-        if isinstance(qualitative_features,str):
+        if isinstance(qualitative_features, str):
             feature_name = [qualitative_features]
 
         for feature_name in qualitative_features:
-            dummies_df = df[df_features.get_dummy_encoded_features()[feature_name]]
+            dummies_df = df[
+                df_features.get_dummy_encoded_features()[feature_name]]
+            dummies_columns = dummies_df.columns.to_list()
 
-            # Create new series from dummy data
-            tmp = dummies_df[dummies_df == 1].stack().reset_index()
-            new_series = []
-            index_count = 0
-            for val in range(0, df.shape[0]):
-                if val not in set(tmp["level_0"]):
-                    new_series.append(np.nan)
-                else:
-                    new_series.append(tmp["level_1"][index_count])
-                    index_count += 1
+            tmp_df = dummies_df[dummies_df == 1].stack().reset_index()
+            del dummies_df
 
-            # -----
-            new_series = pd.Series(new_series).str.replace(feature_name + "_", "")
-            df[feature_name] = new_series
+            df[feature_name] = np.full([len(df)], np.nan)
+            df[feature_name].iloc[tmp_df["level_0"]] = tmp_df[
+                "level_1"].values.tolist()
 
             # Remove dummy features
-            df.drop(columns=dummies_df.columns.to_list(),
+            df.drop(columns=dummies_columns,
                     inplace=True)
 
+            df[feature_name] = df[feature_name].str[len(feature_name) + 1:]
+
             # Remove dummy encoded relationship
             df_features.remove_feature_from_dummy_encoded(feature_name)
 
             # Add feature back to original set in df_features
             try:
-                pd.to_numeric(new_series.dropna())
+                pd.to_numeric(df[feature_name].dropna())
                 df_features.add_new_categorical_feature(feature_name)
 
             except ValueError:
                 df_features.add_new_string_feature(feature_name)
 
-        df.index = old_indexes
         if _add_to_que:
             params_dict = locals()
             parameters = get_parameters(self.revert_dummies)
@@ -467,7 +460,7 @@ class DataEncoder(DataPipelineSegment):
     def __bool_string_values_check(self,
                                    feature_values):
         """
-        Desc:
+
             Checks if a collection of strings can be considered a bool feature
             based on the amount of strings and the values of those strings.
 
