@@ -5,6 +5,7 @@ from eflow._hidden.parent_objects import ModelAnalysis
 from eflow._hidden.custom_exceptions import RequiresPredictionMethods, ProbasNotPossible, \
     UnsatisfiedRequirments
 from eflow.data_analysis import FeatureAnalysis
+from eflow.data_pipeline_segments import DataEncoder
 
 from eflow._hidden.constants import GRAPH_DEFAULTS
 
@@ -151,6 +152,12 @@ class ClassificationAnalysis(ModelAnalysis):
                                   self.folder_path + "_Extras",
                                   "_Classes")
 
+        # Save feature order
+        write_object_text_to_file(self.__feature_order,
+                                  self.folder_path + "_Extras",
+                                  "_Feature_Order")
+
+
         # Save features and or df_features object
         df_features.create_json_file_representation(self.folder_path + "_Extras",
                                                     "df_features.json")
@@ -170,7 +177,10 @@ class ClassificationAnalysis(ModelAnalysis):
         # Find the 'type' of each prediction. Probabilities or Predictions
         if self.__pred_funcs_dict:
             for pred_name, pred_func in self.__pred_funcs_dict.items():
-                model_output = pred_func(self.__sample_data)[0]
+                try:
+                    model_output = pred_func(self.__sample_data)[0]
+                except Exception as e:
+                    model_output = pred_func(np.array(self.__sample_data))[0]
 
                 # Confidence / Probability (Continuous output)
                 if isinstance(model_output, list) or isinstance(model_output,
@@ -212,7 +222,7 @@ class ClassificationAnalysis(ModelAnalysis):
                                           "weighted"],
                          display_visuals=True):
         """
-        Desc:
+
             Runs all available analysis functions on the models predicted data.
 
         Args:
@@ -444,7 +454,7 @@ class ClassificationAnalysis(ModelAnalysis):
                           text_fontsize='medium'):
 
         """
-        Desc:
+
             From scikit-plot documentation
             Link: http://tinyurl.com/y3ym5pyc
             Generates the KS Statistic plot from labels and scores/probabilities.
@@ -492,6 +502,10 @@ class ClassificationAnalysis(ModelAnalysis):
                                         figsize=figsize,
                                         title_fontsize=title_fontsize,
                                         text_fontsize=text_fontsize)
+        legend = plt.legend(frameon=1)
+        frame = legend.get_frame()
+        frame.set_facecolor('white')
+        frame.set_edgecolor('white')
         if save_file:
             self.save_plot(filename=filename,
                            sub_dir=sub_dir)
@@ -519,7 +533,7 @@ class ClassificationAnalysis(ModelAnalysis):
                        text_fontsize='medium'):
 
         """
-        Desc:
+
             From scikit-plot documentation
             Link: http://tinyurl.com/y3ym5pyc
             Creates ROC curves from labels and predicted probabilities.
@@ -567,6 +581,12 @@ class ClassificationAnalysis(ModelAnalysis):
                                title_fontsize=title_fontsize,
                                text_fontsize=text_fontsize)
 
+        legend = plt.legend(frameon=1)
+        frame = legend.get_frame()
+        frame.set_facecolor('white')
+        frame.set_edgecolor('white')
+
+
         if save_file:
             self.save_plot(filename=filename,
                            sub_dir=sub_dir)
@@ -593,7 +613,7 @@ class ClassificationAnalysis(ModelAnalysis):
                              text_fontsize='medium'):
 
         """
-        Desc:
+
             From scikit-plot documentation
             Link: http://tinyurl.com/y3ym5pyc
             Plots calibration curves for a set of classifier probability estimates.
@@ -641,6 +661,12 @@ class ClassificationAnalysis(ModelAnalysis):
                                            title_fontsize=title_fontsize,
                                            text_fontsize=text_fontsize)
 
+        legend = plt.legend(frameon=1)
+        frame = legend.get_frame()
+        frame.set_facecolor('white')
+        frame.set_edgecolor('white')
+
+
         if save_file:
             self.save_plot(filename=filename,
                            sub_dir=sub_dir)
@@ -670,7 +696,7 @@ class ClassificationAnalysis(ModelAnalysis):
                                     title_fontsize='large',
                                     text_fontsize='medium'):
         """
-        Desc:
+
             From scikit-plot documentation
             Link: http://tinyurl.com/y3ym5pyc
             Plots precision recall curve plot based on the models predictions.
@@ -722,6 +748,11 @@ class ClassificationAnalysis(ModelAnalysis):
                                             title_fontsize=title_fontsize,
                                             text_fontsize=text_fontsize)
 
+        legend = plt.legend(frameon=1)
+        frame = legend.get_frame()
+        frame.set_facecolor('white')
+        frame.set_edgecolor('white')
+
         if save_file:
             self.save_plot(filename=filename,
                            sub_dir=sub_dir)
@@ -748,7 +779,7 @@ class ClassificationAnalysis(ModelAnalysis):
                         title_fontsize='large',
                         text_fontsize='medium'):
         """
-        Desc:
+
             From scikit-plot documentation
             Link: http://tinyurl.com/y3ym5pyc
             The lift curve is used to determine the effectiveness of a binary classifier.
@@ -799,6 +830,12 @@ class ClassificationAnalysis(ModelAnalysis):
                                       title_fontsize=title_fontsize,
                                       text_fontsize=text_fontsize)
 
+
+        legend = plt.legend(frameon=1)
+        frame = legend.get_frame()
+        frame.set_facecolor('white')
+        frame.set_edgecolor('white')
+
         if save_file:
             self.save_plot(filename=filename,
                            sub_dir=sub_dir)
@@ -830,7 +867,7 @@ class ClassificationAnalysis(ModelAnalysis):
                               title_fontsize='large',
                               text_fontsize='medium'):
         """
-        Desc:
+
             From scikit-plot documentation
             Link: http://tinyurl.com/y3ym5pyc
             Creates a confusion matrix plot based on the models predictions.
@@ -913,7 +950,7 @@ class ClassificationAnalysis(ModelAnalysis):
                                                 "macro",
                                                 "weighted"]):
         """
-        Desc:
+
             Creates a dataframe based on the prediction metrics
             of the feature matrix and target vector.
 
@@ -1067,7 +1104,7 @@ class ClassificationAnalysis(ModelAnalysis):
                                         extra_tables=True,
                                         statistical_analysis_on_aggregates=True):
         """
-        Desc:
+
             Compares the actual target value to the predicted value and performs
             analysis of all the data.
 
@@ -1166,8 +1203,24 @@ class ClassificationAnalysis(ModelAnalysis):
                                  sub_dir + "/Correctly Predicted Data/All Correct Data")
             output_path = f"{self.folder_path}/{sub_dir}/Correctly Predicted Data"
 
+            tmp_df_features = copy.deepcopy(self.__df_features)
+
+            data_encoder = DataEncoder(create_file=False)
+
+            data_encoder.revert_dummies(correct_df,
+                                        tmp_df_features,
+                                        qualitative_features=list(self.__df_features.get_dummy_encoded_features().keys()))
+
+            data_encoder.decode_data(correct_df,
+                                     tmp_df_features,
+                                     apply_value_representation=False)
+
+            data_encoder.apply_value_representation(correct_df,
+                                                    tmp_df_features)
+            del data_encoder
+
             # Create feature analysis
-            feature_analysis = FeatureAnalysis(self.__df_features,
+            feature_analysis = FeatureAnalysis(tmp_df_features,
                                                overwrite_full_path=output_path + "/All Correct Data")
             feature_analysis.perform_analysis(correct_df,
                                               dataset_name=dataset_name,
@@ -1197,7 +1250,7 @@ class ClassificationAnalysis(ModelAnalysis):
                         if tmp_df.shape[0]:
                             # Create feature analysis directory structure with given graphics
                             feature_analysis = FeatureAnalysis(
-                                self.__df_features,
+                                tmp_df_features,
                                 overwrite_full_path=f"/Actual and Predicted:{pred_target}")
                             feature_analysis.perform_analysis(tmp_df,
                                                               dataset_name=dataset_name,
@@ -1228,7 +1281,7 @@ class ClassificationAnalysis(ModelAnalysis):
                                       extra_tables=True,
                                       statistical_analysis_on_aggregates=True):
         """
-        Desc:
+
             Compares the actual target value to the predicted value and performs
             analysis of all the data.
 
@@ -1320,14 +1373,32 @@ class ClassificationAnalysis(ModelAnalysis):
             error_df = pd.DataFrame.from_records(X[model_predictions != y])
             error_df.columns = self.__feature_order
             error_df[self.__target_feature] = y[model_predictions != y]
+            error_df.reset_index(drop=True,
+                                 inplace=True)
 
             # Directory path
             create_dir_structure(self.folder_path,
                                  sub_dir + "/Incorrectly Predicted Data/All Incorrect Data")
             output_path = f"{self.folder_path}/{sub_dir}/Incorrectly Predicted Data"
 
+            tmp_df_features = copy.deepcopy(self.__df_features)
+
+            data_encoder = DataEncoder(create_file=False)
+
+            data_encoder.revert_dummies(error_df,
+                                        tmp_df_features,
+                                        qualitative_features=list(self.__df_features.get_dummy_encoded_features().keys()))
+
+            data_encoder.decode_data(error_df,
+                                     tmp_df_features,
+                                     apply_value_representation=False)
+
+            data_encoder.apply_value_representation(error_df,
+                                                    tmp_df_features)
+            del data_encoder
+
             # Create feature analysis
-            feature_analysis = FeatureAnalysis(self.__df_features,
+            feature_analysis = FeatureAnalysis(tmp_df_features,
                                                overwrite_full_path=output_path + "/All Incorrect Data")
             feature_analysis.perform_analysis(error_df,
                                               dataset_name=dataset_name,
@@ -1363,9 +1434,8 @@ class ClassificationAnalysis(ModelAnalysis):
                                         inplace=True)
                             if tmp_df.shape[0]:
                                 # Create feature analysis directory structure with given graphics
-                                feature_analysis = FeatureAnalysis(
-                                    self.__df_features,
-                                    overwrite_full_path=f"{output_path}/Predicted:{pred_target} Actual: {actual_target}")
+                                feature_analysis = FeatureAnalysis(tmp_df_features,
+                                                                   overwrite_full_path=f"{output_path}/Predicted:{pred_target} Actual: {actual_target}")
                                 feature_analysis.perform_analysis(tmp_df,
                                                                   dataset_name=dataset_name,
                                                                   target_features=[
@@ -1389,7 +1459,7 @@ class ClassificationAnalysis(ModelAnalysis):
                               display_visuals=True,
                               save_file=True):
         """
-        Desc:
+
             Creates a report of all target's metric evaluations
             based on the model's prediction output from the classification report
             from the sklearn.
@@ -1501,7 +1571,7 @@ class ClassificationAnalysis(ModelAnalysis):
                                X,
                                thresholds=None):
         """
-        Desc:
+
             Finds the model's predicted labels.
 
         Args:
@@ -1565,7 +1635,7 @@ class ClassificationAnalysis(ModelAnalysis):
                            pred_name,
                            X):
         """
-        Desc:
+
             Attempts to get the probabilities from the prediction function.
 
         Args:
@@ -1599,7 +1669,7 @@ class ClassificationAnalysis(ModelAnalysis):
                                          dataset_name,
                                          thresholds):
         """
-        Desc:
+
             Iterates through directory structure looking at each text file
             containing a string representation of the given threshold;
             keeps comparing the passed value of 'thresholds' to the text file.
@@ -1669,7 +1739,7 @@ class ClassificationAnalysis(ModelAnalysis):
                                                  directory_path,
                                                  thresholds):
         """
-        Desc:
+
             Compare the thresholds object to a threshold text file found in
             the directory; returns true if the file exists and the object's
             value matches up.
